﻿using Microsoft.AspNet.Identity;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace THEasy.Model
{
    [Table("User")]
    public class User : IUser<int>
    {
        [Key]
        public int Id { get; set; }

        [Required]
        public string UserName { get; set; }

        [DataType(DataType.Password)]
        public string Password { get; set; }

        public string Email { get; set; }

        [Required]
        public string FirstName { get; set; }

        [Required]
        public string LastName { get; set; }

        public string FullName => LastName + " " + FirstName;

        [Display(Name = "Active Directory User")]
        public bool IsAdUser { get; set; }

        public bool IsDeleted { get; set; }

        public bool IsDisabled { get; set; }

        public DateTimeOffset LockoutDate { get; set; }

        [NotMapped]
        private bool _isLocked;
        public bool IsLocked {
            get
            {
                return LockoutDate > DateTimeOffset.Now;
            }
            set
            {
                _isLocked = value;
                if(value)
                {
                    LockoutDate = DateTimeOffset.MaxValue;
                }
                else
                {
                    LockoutDate = DateTimeOffset.MinValue;
                }
            }
        }    

        public bool ChangePasswordNextLogon { get; set; }
        public int AccessFailedCount { get; set; }

        public virtual ICollection<Group> Groups { get; set; }

        public virtual ICollection<Access> Accesses { get; set; }
        public virtual ICollection<ACL> ACLs { get; set; }

    }
}
