﻿using Microsoft.AspNet.Identity;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Data.Entity.Migrations;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using THEasy.Model;

namespace THEasy.Authentication
{
    public class UserStore : IUserStore<User, int>,
        IUserPasswordStore<User, int>,
        IUserLockoutStore<User, int>
    {
        private readonly DB.THEasyEntities _db = new DB.THEasyEntities();

        public void Dispose()
        {
            _db.Dispose();
        }
        public async Task<User> FindByIdAsync(int userId)
        {
            return await _db.Users.FirstOrDefaultAsync(a => a.Id == userId && !a.IsDeleted && !a.IsDisabled);
        }

        public async Task<User> FindByNameAsync(string username)
        {
            return await _db.Users.FirstOrDefaultAsync(a => a.UserName == username && !a.IsDeleted && !a.IsDisabled);
        }

        public async Task CreateAsync(User user)
        {
            var item = await _db.Users.FirstOrDefaultAsync(a => a.UserName == user.UserName && a.IsDeleted == false);
            if (item != null)
                throw new Exception("Duplicated username.");

            _db.Users.Add(user);

            await _db.SaveChangesAsync();
        }

        public async Task UpdateAsync(User user)
        {
            if(user.IsAdUser)
            {
                user.Password = null;
                user.LockoutDate = DateTime.MinValue;
                user.AccessFailedCount = 0;
                user.ChangePasswordNextLogon = false;
            }

            DbEntityEntry entry = _db.Entry(user);
            entry.State = EntityState.Modified;

            await _db.SaveChangesAsync();
        }

        public async Task DeleteAsync(User user)
        {
            user.IsDisabled = true;
            user.IsDeleted = true;

            DbEntityEntry entry = _db.Entry(user);
            entry.State = EntityState.Modified;

            await _db.SaveChangesAsync();
        }

        public async Task SetPasswordHashAsync(User user, string passwordHash)
        {
            if (!user.IsAdUser)
            {
                user.Password = passwordHash;

                DbEntityEntry entry = _db.Entry(user);
                entry.State = EntityState.Modified;

                await _db.SaveChangesAsync();
            }
        }

        // ReSharper disable once CSharpWarnings::CS1998
        public async Task<string> GetPasswordHashAsync(User user)
        {
            return await Task.FromResult(user != null && !user.IsAdUser ? user.Password : "");
        }

        public async Task<bool> HasPasswordAsync(User user)
        {
            return await Task.FromResult(!string.IsNullOrWhiteSpace(user.Password));
        }

        public async Task<DateTimeOffset> GetLockoutEndDateAsync(User user)
        {
            return await Task.FromResult<DateTimeOffset>(user.LockoutDate);
        }

        public async Task SetLockoutEndDateAsync(User user, DateTimeOffset lockoutEnd)
        {
            user.LockoutDate = lockoutEnd;

            DbEntityEntry entry = _db.Entry(user);
            entry.State = EntityState.Modified;

            await _db.SaveChangesAsync();
        }

        public async Task<int> IncrementAccessFailedCountAsync(User user)
        {
            user.AccessFailedCount++;

            DbEntityEntry entry = _db.Entry(user);
            entry.State = EntityState.Modified;

            await _db.SaveChangesAsync();

            return await Task.FromResult(user.AccessFailedCount);
        }

        public async Task ResetAccessFailedCountAsync(User user)
        {
            user.AccessFailedCount = 0;

            DbEntityEntry entry = _db.Entry(user);
            entry.State = EntityState.Modified;

            await _db.SaveChangesAsync();
        }

        public async Task<int> GetAccessFailedCountAsync(User user)
        {
            return await Task.FromResult<int>(user.AccessFailedCount);
        }

        public Task<bool> GetLockoutEnabledAsync(User user)
        {
            throw new NotImplementedException();
        }

        public Task SetLockoutEnabledAsync(User user, bool enabled)
        {
            throw new NotImplementedException();
        }
    }
}