﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;

namespace THEasy.Controllers.API
{
    public class AccessController : ApiController, IDisposable
    {
        private DB.THEasyEntities _db = new DB.THEasyEntities();

        [Route("api/access/{objectTag}")]
        public async Task<bool> Get(string objectTag, string username)
        {
            return await _db.Accesses.AnyAsync(a => a.Object.Tag == objectTag 
                && a.User.UserName == username && !a.IsBlocked && a.CanEnter
                && a.DateFrom <= DateTime.Now && (a.DateTo == null || a.DateTo >= DateTime.Now));
        }

        [HttpPost]
        public async Task<IHttpActionResult> Post(string objectTag, string username, DateTime? from, DateTime? to, bool canEnter, bool isBlocked = false, bool isSuspect = false)
        {
            var tmp = await _db.Accesses.FirstOrDefaultAsync(a => a.Object.Tag == objectTag && a.User.UserName == username);
            if(tmp == null)
            {
                var user = await _db.Users.FirstOrDefaultAsync(a => a.UserName == username);
                if (user == null)
                {
                    return BadRequest("User not found.");
                }

                var obj = await _db.Objects.FirstOrDefaultAsync(a => a.Tag == objectTag);
                if(obj == null)
                {
                    return BadRequest("Object not found.");
                }

                tmp = new Model.Access
                {
                    Object = obj,
                    User = user,
                };
                _db.Accesses.Add(tmp);
            }

            tmp.DateFrom = from ?? DateTime.Now;
            tmp.DateTo = to;
            tmp.CanEnter = canEnter;
            tmp.IsBlocked = isBlocked;
            tmp.IsSuspect = isSuspect;

            await _db.SaveChangesAsync();

            return Ok();
        }

        [HttpDelete]
        public async Task<IHttpActionResult> Delete(int id)
        {
            var tmp = await _db.Accesses.FindAsync(id);
            if (tmp == null)
            {
                return NotFound();
            }

            _db.Accesses.Remove(tmp);
            await _db.SaveChangesAsync();

            return Ok();
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }
    }
}
