﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using THEasy.Model;

namespace THEasy.Controllers.API
{
    public class CRUDController : ApiController
    {
        private DB.THEasyEntities _db = new DB.THEasyEntities();

        [Route("api/crud/{objectTag}")]
        public async Task<IEnumerable<ACL>> Get(string objectTag, string username)
        {
            var acluser = await FilterACL(objectTag).Where(a => a.User.UserName == username).ToListAsync();

            var groups = _db.Users.Where(a => a.UserName == username).SelectMany(a => a.Groups).Select(a => a.Name);

            // carico tutti gli ACL di tutti i gruppi di cui l'utente fa parte e prendo il massimo per ogni permesso facendo l'OR come somma.
            var aclgroups = (await FilterACL(objectTag).Where(a => groups.Contains(a.Group.Name)).ToListAsync())
                .GroupBy(a => a.Object.Tag)
                .Select(a => new
                {
                    objectTag = a.Key,
                    create = a.Sum(b => b.Create ? 1 : 0),
                    read = a.Sum(b => b.Read ? 1 : 0),
                    update = a.Sum(b => b.Update ? 1 : 0),
                    delete = a.Sum(b => b.Delete ? 1 : 0)
                }).Select(a => new ACL
                {
                    Object = new EntityObject { Name = a.objectTag },
                    Create = a.create > 1,
                    Read = a.read > 1,
                    Update = a.update > 1,
                    Delete = a.delete > 1
                }).ToList();

            // le ACL su singolo utente vincono su quelle di gruppo a parità di tag, quindi rimuovo quelle di gruppo.
            aclgroups.RemoveAll(a => acluser.Select(b => b.Object.Tag).Contains(a.Object.Tag));

            return aclgroups.Union(acluser);
        }

        [HttpPost]
        [Route("api/crud/{objectTag}")]
        public async Task<IHttpActionResult> Post(string objectTag, bool create, bool read, bool update, bool delete, DateTime? from, DateTime? to, string username = "", string groupname = "")
        {
            var acl = await _db.ACLs.FirstOrDefaultAsync(a => a.Object.Tag == objectTag && (a.User.UserName == username || a.Group.Name == groupname));
            if(acl == null)
            {
                var obj = await _db.Objects.FirstOrDefaultAsync(a => a.Tag == objectTag);
                var user = await _db.Users.FirstOrDefaultAsync(a => a.UserName == username);
                if (user == null && !string.IsNullOrWhiteSpace(username))
                {
                    return BadRequest("User not found.");
                }

                var group = await _db.Groups.FirstOrDefaultAsync(a => a.Name == groupname);

                acl = new ACL
                {
                    Object = obj,
                    User = user,
                    Group = group
                };
                _db.ACLs.Add(acl);
            }

            acl.Create = create;
            acl.Read = read;
            acl.Update = update;
            acl.Delete = delete;
            acl.DateFrom = from ?? DateTime.Now;
            acl.DateTo = to;

            await _db.SaveChangesAsync();

            return Ok();
        }

        private IQueryable<ACL> FilterACL(string objectTag)
        {
            return _db.ACLs.Where(a => (a.Object.Tag == objectTag || a.Object.Parent.Tag == objectTag)
               && a.DateFrom <= DateTime.Now && (a.DateTo == null || a.DateTo >= DateTime.Now));
        }
    }
}
