﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using System.Linq.Dynamic;
using THEasy.DB;
using System.Data.Entity;
using System.Data.Entity.Migrations;
using System.Web.Http.Results;
using System.Web.OData;
using System.Data.Entity.Infrastructure;
using System.Web.OData.Routing;

namespace THEasy.Controllers.API
{
    public class GroupsController : ODataController
    {
        private readonly THEasyEntities _db = new THEasyEntities();

        [EnableQuery]
        public IHttpActionResult Get()
        {
            return Ok(_db.Groups.Where(a => !a.IsDeleted));
        }

        [HttpPost]
        public async Task<IHttpActionResult> Post(Model.Group obj)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            _db.Groups.AddOrUpdate(obj);
            await _db.SaveChangesAsync();

            return Created(obj);
        }

        public async Task<IHttpActionResult> Patch([FromODataUri] int key, Delta<Model.Group> obj)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            var entity = await _db.Groups.FindAsync(key);
            if (entity == null)
            {
                return NotFound();
            }

            obj.Patch(entity);
            try
            {
                await _db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!_db.Groups.Any(p => !p.IsDeleted && p.Id == key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }
            return Updated(entity);
        }

        [HttpDelete]
        public async Task<IHttpActionResult> Delete([FromODataUri] int key)
        {
            var dbinstance = await _db.Groups.FindAsync(key);
            if (dbinstance == null)
            {
                return NotFound();
            }

            dbinstance.IsDeleted = true;
            await _db.SaveChangesAsync();

            return StatusCode(HttpStatusCode.NoContent);
        }

        [EnableQuery]
        public IQueryable<Model.User> GetUsers([FromODataUri] int key)
        {
            return _db.Groups.Where(m => m.Id == key).SelectMany(m => m.Users);
        }

        protected override void Dispose(bool disposing)
        {
            _db.Dispose();
            base.Dispose(disposing);
        }
    }
}
