﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using System.Linq.Dynamic;
using THEasy.DB;
using System.Data.Entity;
using System.Data.Entity.Migrations;
using System.Web.Http.Results;
using System.Web.OData;
using System.Data.Entity.Infrastructure;

namespace THEasy.Controllers.API
{
    public class ObjectsController : ODataController
    {
        private readonly THEasyEntities _db = new THEasyEntities();

        [EnableQuery]
        public IHttpActionResult Get()
        {
            return Ok(_db.Objects);
        }

        [HttpPost]
        public async Task<IHttpActionResult> Post(Model.EntityObject obj)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            _db.Objects.AddOrUpdate(obj);
            await _db.SaveChangesAsync();

            return Created(obj);
        }

        public async Task<IHttpActionResult> Patch([FromODataUri] int key, Delta<Model.EntityObject> obj)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            var entity = await _db.Objects.FindAsync(key);
            if (entity == null)
            {
                return NotFound();
            }

            obj.Patch(entity);
            try
            {
                await _db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!_db.Objects.Any(p => p.Id == key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }
            return Updated(entity);
        }

        [HttpDelete]
        public async Task<IHttpActionResult> Delete([FromODataUri] int key)
        {
            var dbinstance = await _db.Objects.FindAsync(key);
            if (dbinstance == null)
            {
                return NotFound();
            }
            _db.Objects.Remove(dbinstance);
            await _db.SaveChangesAsync();

            return StatusCode(HttpStatusCode.NoContent);
        }

        protected override void Dispose(bool disposing)
        {
            _db.Dispose();
            base.Dispose(disposing);
        }
    }
}
