﻿using System;
using System.Collections.Generic;
using System.Data.Entity.Infrastructure;
using System.Data.Entity.Migrations;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.OData;
using THEasy.DB;

namespace THEasy.Controllers.API
{
    public class UsersController : ODataController
    {
        private readonly THEasyEntities _db = new THEasyEntities();

        [EnableQuery]
        public IHttpActionResult Get()
        {
            return Ok(_db.Users.Where(a=> !a.IsDeleted));
        }

        [HttpPost]
        public async Task<IHttpActionResult> Post(Model.User obj)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            _db.Users.AddOrUpdate(obj);
            await _db.SaveChangesAsync();

            return Created(obj);
        }

        public async Task<IHttpActionResult> Patch([FromODataUri] int key, Delta<Model.User> obj)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            var entity = await _db.Users.FindAsync(key);
            if (entity == null)
            {
                return NotFound();
            }

            obj.Patch(entity);
            try
            {
                await _db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!_db.Users.Any(p => !p.IsDeleted && p.Id == key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }
            return Updated(entity);
        }

        [HttpDelete]
        public async Task<IHttpActionResult> Delete([FromODataUri] int key)
        {
            var dbinstance = await _db.Users.FindAsync(key);
            if (dbinstance == null)
            {
                return NotFound();
            }

            dbinstance.IsDeleted = true;
            await _db.SaveChangesAsync();

            return StatusCode(HttpStatusCode.NoContent);
        }

        protected override void Dispose(bool disposing)
        {
            _db.Dispose();
            base.Dispose(disposing);
        }

    }
}
